@@ -2,19 +2,20 @@
 
 from asyncio import run_coroutine_threadsafe
 import logging
+from typing import Any
 
 import homeconnect
 from homeconnect.api import HomeConnectError
 
 from homeassistant import config_entries, core
+from homeassistant.components.sensor import SensorDeviceClass
 from homeassistant.const import (
     ATTR_DEVICE_CLASS,
     ATTR_ICON,
     CONF_DEVICE,
     CONF_ENTITIES,
-    DEVICE_CLASS_TIMESTAMP,
     PERCENTAGE,
-    TIME_SECONDS,
+    UnitOfTime,
 )
 from homeassistant.helpers import config_entry_oauth2_flow
 from homeassistant.helpers.dispatcher import dispatcher_send
@@ -54,7 +55,7 @@ class ConfigEntryAuth(homeconnect.HomeConnectAPI):
             hass, config_entry, implementation
         )
         super().__init__(self.session.token)
-        self.devices = []
+        self.devices: list[dict[str, Any]] = []
 
     def refresh_tokens(self) -> dict:
         """Refresh and return new Home Connect tokens using Home Assistant OAuth2 session."""
@@ -77,6 +78,10 @@ class ConfigEntryAuth(homeconnect.HomeConnectAPI):
                 device = Dishwasher(self.hass, app)
             elif app.type == "FridgeFreezer":
                 device = FridgeFreezer(self.hass, app)
+            elif app.type == "Refrigerator":
+                device = Refrigerator(self.hass, app)
+            elif app.type == "Freezer":
+                device = Freezer(self.hass, app)
             elif app.type == "Oven":
                 device = Oven(self.hass, app)
             elif app.type == "CoffeeMaker":
@@ -85,6 +90,8 @@ class ConfigEntryAuth(homeconnect.HomeConnectAPI):
                 device = Hood(self.hass, app)
             elif app.type == "Hob":
                 device = Hob(self.hass, app)
+            elif app.type == "CookProcessor":
+                device = CookProcessor(self.hass, app)
             else:
                 _LOGGER.warning("Appliance type %s not implemented", app.type)
                 continue
@@ -138,11 +145,14 @@ class HomeConnectDevice:
 class DeviceWithPrograms(HomeConnectDevice):
     """Device with programs."""
 
-    PROGRAMS = []
-
     def get_programs_available(self):
         """Get the available programs."""
-        return self.PROGRAMS
+        try:
+            programs_available = self.appliance.get_programs_available()
+        except (HomeConnectError, ValueError):
+            _LOGGER.debug("Unable to fetch available programs. Probably offline")
+            programs_available = None
+        return programs_available
 
     def get_program_switches(self):
         """Get a dictionary with info about program switches.
@@ -150,7 +160,7 @@ class DeviceWithPrograms(HomeConnectDevice):
         There will be one switch for each program.
         """
         programs = self.get_programs_available()
-        return [{ATTR_DEVICE: self, "program_name": p["name"]} for p in programs]
+        return [{ATTR_DEVICE: self, "program_name": p} for p in programs]
 
     def get_program_sensors(self):
         """Get a dictionary with info about program sensors.
@@ -159,8 +169,8 @@ class DeviceWithPrograms(HomeConnectDevice):
         device.
         """
         sensors = {
-            "Remaining Program Time": (None, None, DEVICE_CLASS_TIMESTAMP, 1),
-            "Duration": (TIME_SECONDS, "mdi:update", None, 1),
+            "Remaining Program Time": (None, None, SensorDeviceClass.TIMESTAMP, 1),
+            "Duration": (UnitOfTime.SECONDS, "mdi:update", None, 1),
             "Program Progress": (PERCENTAGE, "mdi:progress-clock", None, 1),
         }
         return [
@@ -258,25 +268,6 @@ class Dryer(
 ):
     """Dryer class."""
 
-    PROGRAMS = [
-        {"name": "LaundryCare.Dryer.Program.Cotton"},
-        {"name": "LaundryCare.Dryer.Program.Synthetic"},
-        {"name": "LaundryCare.Dryer.Program.Mix"},
-        {"name": "LaundryCare.Dryer.Program.Blankets"},
-        {"name": "LaundryCare.Dryer.Program.BusinessShirts"},
-        {"name": "LaundryCare.Dryer.Program.DownFeathers"},
-        {"name": "LaundryCare.Dryer.Program.Hygiene"},
-        {"name": "LaundryCare.Dryer.Program.Jeans"},
-        {"name": "LaundryCare.Dryer.Program.Outdoor"},
-        {"name": "LaundryCare.Dryer.Program.SyntheticRefresh"},
-        {"name": "LaundryCare.Dryer.Program.Towels"},
-        {"name": "LaundryCare.Dryer.Program.Delicates"},
-        {"name": "LaundryCare.Dryer.Program.Super40"},
-        {"name": "LaundryCare.Dryer.Program.Shirts15"},
-        {"name": "LaundryCare.Dryer.Program.Pillow"},
-        {"name": "LaundryCare.Dryer.Program.AntiShrink"},
-    ]
-
     def get_entity_info(self):
         """Get a dictionary with infos about the associated entities."""
         door_entity = self.get_door_entity()
@@ -302,31 +293,6 @@ class Dishwasher(
 ):
     """Dishwasher class."""
 
-    PROGRAMS = [
-        {"name": "Dishcare.Dishwasher.Program.Auto1"},
-        {"name": "Dishcare.Dishwasher.Program.Auto2"},
-        {"name": "Dishcare.Dishwasher.Program.Auto3"},
-        {"name": "Dishcare.Dishwasher.Program.Eco50"},
-        {"name": "Dishcare.Dishwasher.Program.Quick45"},
-        {"name": "Dishcare.Dishwasher.Program.Intensiv70"},
-        {"name": "Dishcare.Dishwasher.Program.Normal65"},
-        {"name": "Dishcare.Dishwasher.Program.Glas40"},
-        {"name": "Dishcare.Dishwasher.Program.GlassCare"},
-        {"name": "Dishcare.Dishwasher.Program.NightWash"},
-        {"name": "Dishcare.Dishwasher.Program.Quick65"},
-        {"name": "Dishcare.Dishwasher.Program.Normal45"},
-        {"name": "Dishcare.Dishwasher.Program.Intensiv45"},
-        {"name": "Dishcare.Dishwasher.Program.AutoHalfLoad"},
-        {"name": "Dishcare.Dishwasher.Program.IntensivPower"},
-        {"name": "Dishcare.Dishwasher.Program.MagicDaily"},
-        {"name": "Dishcare.Dishwasher.Program.Super60"},
-        {"name": "Dishcare.Dishwasher.Program.Kurz60"},
-        {"name": "Dishcare.Dishwasher.Program.ExpressSparkle65"},
-        {"name": "Dishcare.Dishwasher.Program.MachineCare"},
-        {"name": "Dishcare.Dishwasher.Program.SteamFresh"},
-        {"name": "Dishcare.Dishwasher.Program.MaximumCleaning"},
-    ]
-
     def get_entity_info(self):
         """Get a dictionary with infos about the associated entities."""
         door_entity = self.get_door_entity()
@@ -351,14 +317,6 @@ class Oven(
 ):
     """Oven class."""
 
-    PROGRAMS = [
-        {"name": "Cooking.Oven.Program.HeatingMode.PreHeating"},
-        {"name": "Cooking.Oven.Program.HeatingMode.HotAir"},
-        {"name": "Cooking.Oven.Program.HeatingMode.TopBottomHeating"},
-        {"name": "Cooking.Oven.Program.HeatingMode.PizzaSetting"},
-        {"name": "Cooking.Oven.Program.Microwave.600Watt"},
-    ]
-
     power_off_state = BSH_POWER_STANDBY
 
     def get_entity_info(self):
@@ -385,30 +343,6 @@ class Washer(
 ):
     """Washer class."""
 
-    PROGRAMS = [
-        {"name": "LaundryCare.Washer.Program.Cotton"},
-        {"name": "LaundryCare.Washer.Program.Cotton.CottonEco"},
-        {"name": "LaundryCare.Washer.Program.EasyCare"},
-        {"name": "LaundryCare.Washer.Program.Mix"},
-        {"name": "LaundryCare.Washer.Program.DelicatesSilk"},
-        {"name": "LaundryCare.Washer.Program.Wool"},
-        {"name": "LaundryCare.Washer.Program.Sensitive"},
-        {"name": "LaundryCare.Washer.Program.Auto30"},
-        {"name": "LaundryCare.Washer.Program.Auto40"},
-        {"name": "LaundryCare.Washer.Program.Auto60"},
-        {"name": "LaundryCare.Washer.Program.Chiffon"},
-        {"name": "LaundryCare.Washer.Program.Curtains"},
-        {"name": "LaundryCare.Washer.Program.DarkWash"},
-        {"name": "LaundryCare.Washer.Program.Dessous"},
-        {"name": "LaundryCare.Washer.Program.Monsoon"},
-        {"name": "LaundryCare.Washer.Program.Outdoor"},
-        {"name": "LaundryCare.Washer.Program.PlushToy"},
-        {"name": "LaundryCare.Washer.Program.ShirtsBlouses"},
-        {"name": "LaundryCare.Washer.Program.SportFitness"},
-        {"name": "LaundryCare.Washer.Program.Towels"},
-        {"name": "LaundryCare.Washer.Program.WaterProof"},
-    ]
-
     def get_entity_info(self):
         """Get a dictionary with infos about the associated entities."""
         door_entity = self.get_door_entity()
@@ -427,23 +361,6 @@ class Washer(
 class CoffeeMaker(DeviceWithOpState, DeviceWithPrograms, DeviceWithRemoteStart):
     """Coffee maker class."""
 
-    PROGRAMS = [
-        {"name": "ConsumerProducts.CoffeeMaker.Program.Beverage.Espresso"},
-        {"name": "ConsumerProducts.CoffeeMaker.Program.Beverage.EspressoMacchiato"},
-        {"name": "ConsumerProducts.CoffeeMaker.Program.Beverage.Coffee"},
-        {"name": "ConsumerProducts.CoffeeMaker.Program.Beverage.Cappuccino"},
-        {"name": "ConsumerProducts.CoffeeMaker.Program.Beverage.LatteMacchiato"},
-        {"name": "ConsumerProducts.CoffeeMaker.Program.Beverage.CaffeLatte"},
-        {"name": "ConsumerProducts.CoffeeMaker.Program.CoffeeWorld.Americano"},
-        {"name": "ConsumerProducts.CoffeeMaker.Program.Beverage.EspressoDoppio"},
-        {"name": "ConsumerProducts.CoffeeMaker.Program.CoffeeWorld.FlatWhite"},
-        {"name": "ConsumerProducts.CoffeeMaker.Program.CoffeeWorld.Galao"},
-        {"name": "ConsumerProducts.CoffeeMaker.Program.Beverage.MilkFroth"},
-        {"name": "ConsumerProducts.CoffeeMaker.Program.Beverage.WarmMilk"},
-        {"name": "ConsumerProducts.CoffeeMaker.Program.Beverage.Ristretto"},
-        {"name": "ConsumerProducts.CoffeeMaker.Program.CoffeeWorld.Cortado"},
-    ]
-
     power_off_state = BSH_POWER_STANDBY
 
     def get_entity_info(self):
@@ -469,12 +386,6 @@ class Hood(
 ):
     """Hood class."""
 
-    PROGRAMS = [
-        {"name": "Cooking.Common.Program.Hood.Automatic"},
-        {"name": "Cooking.Common.Program.Hood.Venting"},
-        {"name": "Cooking.Common.Program.Hood.DelayedShutOff"},
-    ]
-
     def get_entity_info(self):
         """Get a dictionary with infos about the associated entities."""
         remote_control = self.get_remote_control()
@@ -501,11 +412,27 @@ class FridgeFreezer(DeviceWithDoor):
         return {"binary_sensor": [door_entity]}
 
 
+class Refrigerator(DeviceWithDoor):
+    """Refrigerator class."""
+
+    def get_entity_info(self):
+        """Get a dictionary with infos about the associated entities."""
+        door_entity = self.get_door_entity()
+        return {"binary_sensor": [door_entity]}
+
+
+class Freezer(DeviceWithDoor):
+    """Freezer class."""
+
+    def get_entity_info(self):
+        """Get a dictionary with infos about the associated entities."""
+        door_entity = self.get_door_entity()
+        return {"binary_sensor": [door_entity]}
+
+
 class Hob(DeviceWithOpState, DeviceWithPrograms, DeviceWithRemoteControl):
     """Hob class."""
 
-    PROGRAMS = [{"name": "Cooking.Hob.Program.PowerLevelMode"}]
-
     def get_entity_info(self):
         """Get a dictionary with infos about the associated entities."""
         remote_control = self.get_remote_control()
@@ -517,3 +444,14 @@ class Hob(DeviceWithOpState, DeviceWithPrograms, DeviceWithRemoteControl):
             "switch": program_switches,
             "sensor": program_sensors + op_state_sensor,
         }
+
+
+class CookProcessor(DeviceWithOpState):
+    """CookProcessor class."""
+
+    power_off_state = BSH_POWER_STANDBY
+
+    def get_entity_info(self):
+        """Get a dictionary with infos about the associated entities."""
+        op_state_sensor = self.get_opstate_sensor()
+        return {"sensor": op_state_sensor}
