@@ -1,7 +1,7 @@
 """MQTT component mixins and helpers."""
 from __future__ import annotations
 
-from abc import abstractmethod
+from abc import ABC, abstractmethod
 import asyncio
 from collections.abc import Callable, Coroutine
 from functools import partial
@@ -34,7 +34,10 @@ from homeassistant.helpers import (
     device_registry as dr,
     entity_registry as er,
 )
-from homeassistant.helpers.device_registry import EVENT_DEVICE_REGISTRY_UPDATED
+from homeassistant.helpers.device_registry import (
+    EVENT_DEVICE_REGISTRY_UPDATED,
+    DeviceEntry,
+)
 from homeassistant.helpers.dispatcher import (
     async_dispatcher_connect,
     async_dispatcher_send,
@@ -43,14 +46,13 @@ from homeassistant.helpers.entity import (
     ENTITY_CATEGORIES_SCHEMA,
     DeviceInfo,
     Entity,
-    EntityCategory,
     async_generate_entity_id,
 )
 from homeassistant.helpers.entity_platform import AddEntitiesCallback
 from homeassistant.helpers.event import async_track_entity_registry_updated_event
 from homeassistant.helpers.issue_registry import IssueSeverity, async_create_issue
-from homeassistant.helpers.json import json_loads
 from homeassistant.helpers.typing import ConfigType, DiscoveryInfoType
+from homeassistant.util.json import json_loads
 
 from . import debug_info, subscription
 from .client import async_publish
@@ -74,11 +76,18 @@ from .discovery import (
     MQTT_DISCOVERY_DONE,
     MQTT_DISCOVERY_NEW,
     MQTT_DISCOVERY_UPDATED,
+    MQTTDiscoveryPayload,
     clear_discovery_hash,
     set_discovery_hash,
 )
-from .models import MqttValueTemplate, PublishPayloadType, ReceiveMessage
+from .models import (
+    MqttValueTemplate,
+    PublishPayloadType,
+    ReceiveMessage,
+    ReceivePayloadType,
+)
 from .subscription import (
+    EntitySubscription,
     async_prepare_subscribe_topics,
     async_subscribe_topics,
     async_unsubscribe_topics,
@@ -222,7 +231,7 @@ MQTT_ENTITY_COMMON_SCHEMA = MQTT_AVAILABILITY_SCHEMA.extend(
 )
 
 
-def warn_for_legacy_schema(domain: str) -> Callable:
+def warn_for_legacy_schema(domain: str) -> Callable[[ConfigType], ConfigType]:
     """Warn once when a legacy platform schema is used."""
     warned = set()
 
@@ -230,13 +239,16 @@ def warn_for_legacy_schema(domain: str) -> Callable:
         """Return a validator."""
         nonlocal warned
 
+        # Logged error and repair can be removed from HA 2023.6
         if domain in warned:
             return config
 
-        _LOGGER.warning(
-            "Manually configured MQTT %s(s) found under platform key '%s', "
-            "please move to the mqtt integration key, see "
-            "https://www.home-assistant.io/integrations/%s.mqtt/#new_format",
+        _LOGGER.error(
+            (
+                "Manually configured MQTT %s(s) found under platform key '%s', "
+                "please move to the mqtt integration key, see "
+                "https://www.home-assistant.io/integrations/%s.mqtt/"
+            ),
             domain,
             domain,
             domain,
@@ -249,10 +261,13 @@ def warn_for_legacy_schema(domain: str) -> Callable:
             f"deprecated_yaml_{domain}",
             breaks_in_ha_version="2022.12.0",  # Warning first added in 2022.6.0
             is_fixable=False,
-            severity=IssueSeverity.WARNING,
+            severity=IssueSeverity.ERROR,
             translation_key="deprecated_yaml",
             translation_placeholders={
-                "more_info_url": f"https://www.home-assistant.io/integrations/{domain}.mqtt/#new_format",
+                "more_info_url": (
+                    "https://www.home-assistant.io"
+                    f"/integrations/{domain}.mqtt/#new_format"
+                ),
                 "platform": domain,
             },
         )
@@ -269,53 +284,39 @@ class SetupEntity(Protocol):
         hass: HomeAssistant,
         async_add_entities: AddEntitiesCallback,
         config: ConfigType,
-        config_entry: ConfigEntry | None = None,
-        discovery_data: dict[str, Any] | None = None,
+        config_entry: ConfigEntry,
+        discovery_data: DiscoveryInfoType | None = None,
     ) -> None:
         """Define setup_entities type."""
 
 
-async def async_get_platform_config_from_yaml(
-    hass: HomeAssistant,
-    platform_domain: str,
-    config_yaml: ConfigType | None = None,
-) -> list[ConfigType]:
-    """Return a list of validated configurations for the domain."""
-    mqtt_data = get_mqtt_data(hass)
-    if config_yaml is None:
-        config_yaml = mqtt_data.config
-    if not config_yaml:
-        return []
-    if not (platform_configs := config_yaml.get(platform_domain)):
-        return []
-    return platform_configs
-
-
 async def async_setup_entry_helper(
     hass: HomeAssistant,
     domain: str,
-    async_setup: partial[Coroutine[HomeAssistant, str, None]],
+    async_setup: partial[Coroutine[Any, Any, None]],
     discovery_schema: vol.Schema,
 ) -> None:
     """Set up entity, automation or tag creation dynamically through MQTT discovery."""
     mqtt_data = get_mqtt_data(hass)
 
-    async def async_discover(discovery_payload):
+    async def async_discover(discovery_payload: MQTTDiscoveryPayload) -> None:
         """Discover and add an MQTT entity, automation or tag."""
         if not mqtt_config_entry_enabled(hass):
             _LOGGER.warning(
-                "MQTT integration is disabled, skipping setup of discovered item "
-                "MQTT %s, payload %s",
+                (
+                    "MQTT integration is disabled, skipping setup of discovered item "
+                    "MQTT %s, payload %s"
+                ),
                 domain,
                 discovery_payload,
             )
             return
         discovery_data = discovery_payload.discovery_data
         try:
-            config = discovery_schema(discovery_payload)
+            config: DiscoveryInfoType = discovery_schema(discovery_payload)
             await async_setup(config, discovery_data=discovery_data)
         except Exception:
-            discovery_hash = discovery_data[ATTR_DISCOVERY_HASH]
+            discovery_hash: tuple[str, str] = discovery_data[ATTR_DISCOVERY_HASH]
             clear_discovery_hash(hass, discovery_hash)
             async_dispatcher_send(
                 hass, MQTT_DISCOVERY_DONE.format(discovery_hash), None
@@ -340,48 +341,16 @@ async def async_setup_entry_helper(
             return
         if domain not in config_yaml:
             return
-        await asyncio.gather(
-            *[
-                async_setup(config)
-                for config in await async_get_platform_config_from_yaml(
-                    hass, domain, config_yaml
-                )
-            ]
-        )
+        await asyncio.gather(*[async_setup(config) for config in config_yaml[domain]])
 
     # discover manual configured MQTT items
     mqtt_data.reload_handlers[domain] = _async_setup_entities
     await _async_setup_entities()
 
 
-async def async_setup_platform_helper(
-    hass: HomeAssistant,
-    platform_domain: str,
-    config: ConfigType | DiscoveryInfoType,
-    async_add_entities: AddEntitiesCallback,
-    async_setup_entities: SetupEntity,
+def init_entity_id_from_config(
+    hass: HomeAssistant, entity: Entity, config: ConfigType, entity_id_format: str
 ) -> None:
-    """Help to set up the platform for manual configured MQTT entities."""
-    mqtt_data = get_mqtt_data(hass)
-    if mqtt_data.reload_entry:
-        _LOGGER.debug(
-            "MQTT integration is %s, skipping setup of manually configured MQTT items while unloading the config entry",
-            platform_domain,
-        )
-        return
-    if not (entry_status := mqtt_config_entry_enabled(hass)):
-        _LOGGER.warning(
-            "MQTT integration is %s, skipping setup of manually configured MQTT %s",
-            "not setup" if entry_status is None else "disabled",
-            platform_domain,
-        )
-        return
-    # Ensure we set config_entry when entries are set up to enable clean up
-    config_entry: ConfigEntry = hass.config_entries.async_entries(DOMAIN)[0]
-    await async_setup_entities(hass, async_add_entities, config, config_entry)
-
-
-def init_entity_id_from_config(hass, entity, config, entity_id_format):
     """Set entity_id from object_id if defined in config."""
     if CONF_OBJECT_ID in config:
         entity.entity_id = async_generate_entity_id(
@@ -394,10 +363,10 @@ class MqttAttributes(Entity):
 
     _attributes_extra_blocked: frozenset[str] = frozenset()
 
-    def __init__(self, config: dict) -> None:
+    def __init__(self, config: ConfigType) -> None:
         """Initialize the JSON attributes mixin."""
         self._attributes: dict[str, Any] | None = None
-        self._attributes_sub_state = None
+        self._attributes_sub_state: dict[str, EntitySubscription] = {}
         self._attributes_config = config
 
     async def async_added_to_hass(self) -> None:
@@ -406,16 +375,16 @@ class MqttAttributes(Entity):
         self._attributes_prepare_subscribe_topics()
         await self._attributes_subscribe_topics()
 
-    def attributes_prepare_discovery_update(self, config: dict):
+    def attributes_prepare_discovery_update(self, config: DiscoveryInfoType) -> None:
         """Handle updated discovery message."""
         self._attributes_config = config
         self._attributes_prepare_subscribe_topics()
 
-    async def attributes_discovery_update(self, config: dict):
+    async def attributes_discovery_update(self, config: DiscoveryInfoType) -> None:
         """Handle updated discovery message."""
         await self._attributes_subscribe_topics()
 
-    def _attributes_prepare_subscribe_topics(self):
+    def _attributes_prepare_subscribe_topics(self) -> None:
         """(Re)Subscribe to topics."""
         attr_tpl = MqttValueTemplate(
             self._attributes_config.get(CONF_JSON_ATTRS_TEMPLATE), entity=self
@@ -458,11 +427,11 @@ class MqttAttributes(Entity):
             },
         )
 
-    async def _attributes_subscribe_topics(self):
+    async def _attributes_subscribe_topics(self) -> None:
         """(Re)Subscribe to topics."""
         await async_subscribe_topics(self.hass, self._attributes_sub_state)
 
-    async def async_will_remove_from_hass(self):
+    async def async_will_remove_from_hass(self) -> None:
         """Unsubscribe when removed."""
         self._attributes_sub_state = async_unsubscribe_topics(
             self.hass, self._attributes_sub_state
@@ -477,11 +446,11 @@ class MqttAttributes(Entity):
 class MqttAvailability(Entity):
     """Mixin used for platforms that report availability."""
 
-    def __init__(self, config: dict) -> None:
+    def __init__(self, config: ConfigType) -> None:
         """Initialize the availability mixin."""
-        self._availability_sub_state = None
-        self._available: dict = {}
-        self._available_latest = False
+        self._availability_sub_state: dict[str, EntitySubscription] = {}
+        self._available: dict[str, str | bool] = {}
+        self._available_latest: bool = False
         self._availability_setup_from_config(config)
 
     async def async_added_to_hass(self) -> None:
@@ -498,18 +467,18 @@ class MqttAvailability(Entity):
             )
         )
 
-    def availability_prepare_discovery_update(self, config: dict):
+    def availability_prepare_discovery_update(self, config: DiscoveryInfoType) -> None:
         """Handle updated discovery message."""
         self._availability_setup_from_config(config)
         self._availability_prepare_subscribe_topics()
 
-    async def availability_discovery_update(self, config: dict):
+    async def availability_discovery_update(self, config: DiscoveryInfoType) -> None:
         """Handle updated discovery message."""
         await self._availability_subscribe_topics()
 
-    def _availability_setup_from_config(self, config):
+    def _availability_setup_from_config(self, config: ConfigType) -> None:
         """(Re)Setup."""
-        self._avail_topics = {}
+        self._avail_topics: dict[str, dict[str, Any]] = {}
         if CONF_AVAILABILITY_TOPIC in config:
             self._avail_topics[config[CONF_AVAILABILITY_TOPIC]] = {
                 CONF_PAYLOAD_AVAILABLE: config[CONF_PAYLOAD_AVAILABLE],
@@ -518,6 +487,7 @@ class MqttAvailability(Entity):
             }
 
         if CONF_AVAILABILITY in config:
+            avail: dict[str, Any]
             for avail in config[CONF_AVAILABILITY]:
                 self._avail_topics[avail[CONF_TOPIC]] = {
                     CONF_PAYLOAD_AVAILABLE: avail[CONF_PAYLOAD_AVAILABLE],
@@ -533,7 +503,7 @@ class MqttAvailability(Entity):
 
         self._avail_config = config
 
-    def _availability_prepare_subscribe_topics(self):
+    def _availability_prepare_subscribe_topics(self) -> None:
         """(Re)Subscribe to topics."""
 
         @callback
@@ -541,6 +511,7 @@ class MqttAvailability(Entity):
         def availability_message_received(msg: ReceiveMessage) -> None:
             """Handle a new received MQTT availability message."""
             topic = msg.topic
+            payload: ReceivePayloadType
             payload = self._avail_topics[topic][CONF_AVAILABILITY_TEMPLATE](msg.payload)
             if payload == self._avail_topics[topic][CONF_PAYLOAD_AVAILABLE]:
                 self._available[topic] = True
@@ -555,7 +526,7 @@ class MqttAvailability(Entity):
             topic: (self._available[topic] if topic in self._available else False)
             for topic in self._avail_topics
         }
-        topics = {
+        topics: dict[str, dict[str, Any]] = {
             f"availability_{topic}": {
                 "topic": topic,
                 "msg_callback": availability_message_received,
@@ -571,17 +542,17 @@ class MqttAvailability(Entity):
             topics,
         )
 
-    async def _availability_subscribe_topics(self):
+    async def _availability_subscribe_topics(self) -> None:
         """(Re)Subscribe to topics."""
         await async_subscribe_topics(self.hass, self._availability_sub_state)
 
     @callback
-    def async_mqtt_connect(self):
+    def async_mqtt_connect(self) -> None:
         """Update state on connection/disconnection to MQTT broker."""
         if not self.hass.is_stopping:
             self.async_write_ha_state()
 
-    async def async_will_remove_from_hass(self):
+    async def async_will_remove_from_hass(self) -> None:
         """Unsubscribe when removed."""
         self._availability_sub_state = async_unsubscribe_topics(
             self.hass, self._availability_sub_state
@@ -607,7 +578,11 @@ class MqttAvailability(Entity):
 async def cleanup_device_registry(
     hass: HomeAssistant, device_id: str | None, config_entry_id: str | None
 ) -> None:
-    """Remove MQTT from the device registry entry if there are no remaining entities, triggers or tags."""
+    """Clean up the device registry after MQTT removal.
+
+    Remove MQTT from the device registry entry if there are no remaining
+    entities, triggers or tags.
+    """
     # Local import to avoid circular dependencies
     # pylint: disable-next=import-outside-toplevel
     from . import device_trigger, tag
@@ -628,12 +603,13 @@ async def cleanup_device_registry(
         )
 
 
-def get_discovery_hash(discovery_data: dict) -> tuple[str, str]:
+def get_discovery_hash(discovery_data: DiscoveryInfoType) -> tuple[str, str]:
     """Get the discovery hash from the discovery data."""
-    return discovery_data[ATTR_DISCOVERY_HASH]
+    discovery_hash: tuple[str, str] = discovery_data[ATTR_DISCOVERY_HASH]
+    return discovery_hash
 
 
-def send_discovery_done(hass: HomeAssistant, discovery_data: dict) -> None:
+def send_discovery_done(hass: HomeAssistant, discovery_data: DiscoveryInfoType) -> None:
     """Acknowledge a discovery message has been handled."""
     discovery_hash = get_discovery_hash(discovery_data)
     async_dispatcher_send(hass, MQTT_DISCOVERY_DONE.format(discovery_hash), None)
@@ -641,7 +617,7 @@ def send_discovery_done(hass: HomeAssistant, discovery_data: dict) -> None:
 
 def stop_discovery_updates(
     hass: HomeAssistant,
-    discovery_data: dict,
+    discovery_data: DiscoveryInfoType,
     remove_discovery_updated: Callable[[], None] | None = None,
 ) -> None:
     """Stop discovery updates of being sent."""
@@ -652,15 +628,21 @@ def stop_discovery_updates(
     clear_discovery_hash(hass, discovery_hash)
 
 
-async def async_remove_discovery_payload(hass: HomeAssistant, discovery_data: dict):
-    """Clear retained discovery topic in broker to avoid rediscovery after a restart of HA."""
+async def async_remove_discovery_payload(
+    hass: HomeAssistant, discovery_data: DiscoveryInfoType
+) -> None:
+    """Clear retained discovery payload.
+
+    Remove discovery topic in broker to avoid rediscovery
+    after a restart of Home Assistant.
+    """
     discovery_topic = discovery_data[ATTR_DISCOVERY_TOPIC]
     await async_publish(hass, discovery_topic, "", retain=True)
 
 
 async def async_clear_discovery_topic_if_entity_removed(
     hass: HomeAssistant,
-    discovery_data: dict[str, Any],
+    discovery_data: DiscoveryInfoType,
     event: Event,
 ) -> None:
     """Clear the discovery topic if the entity is removed."""
@@ -669,13 +651,13 @@ async def async_clear_discovery_topic_if_entity_removed(
         await async_remove_discovery_payload(hass, discovery_data)
 
 
-class MqttDiscoveryDeviceUpdate:
+class MqttDiscoveryDeviceUpdate(ABC):
     """Add support for auto discovery for platforms without an entity."""
 
     def __init__(
         self,
         hass: HomeAssistant,
-        discovery_data: dict,
+        discovery_data: DiscoveryInfoType,
         device_id: str | None,
         config_entry: ConfigEntry,
         log_name: str,
@@ -718,7 +700,7 @@ class MqttDiscoveryDeviceUpdate:
 
     async def async_discovery_update(
         self,
-        discovery_payload: DiscoveryInfoType | None,
+        discovery_payload: MQTTDiscoveryPayload,
     ) -> None:
         """Handle discovery update."""
         discovery_hash = get_discovery_hash(self._discovery_data)
@@ -737,8 +719,12 @@ class MqttDiscoveryDeviceUpdate:
                 self.log_name,
                 discovery_hash,
             )
-            await self.async_update(discovery_payload)
-        if not discovery_payload:
+            try:
+                await self.async_update(discovery_payload)
+            finally:
+                send_discovery_done(self.hass, self._discovery_data)
+            self._discovery_data[ATTR_DISCOVERY_PAYLOAD] = discovery_payload
+        elif not discovery_payload:
             # Unregister and clean up the current discovery instance
             stop_discovery_updates(
                 self.hass, self._discovery_data, self._remove_discovery_updated
@@ -789,7 +775,8 @@ class MqttDiscoveryDeviceUpdate:
                 self.hass, self._device_id, self._config_entry_id
             )
 
-    async def async_update(self, discovery_data: dict) -> None:
+    @abstractmethod
+    async def async_update(self, discovery_data: MQTTDiscoveryPayload) -> None:
         """Handle the update of platform specific parts, extend to the platform."""
 
     @abstractmethod
@@ -803,13 +790,14 @@ class MqttDiscoveryUpdate(Entity):
     def __init__(
         self,
         hass: HomeAssistant,
-        discovery_data: dict | None,
-        discovery_update: Callable | None = None,
+        discovery_data: DiscoveryInfoType | None,
+        discovery_update: Callable[[MQTTDiscoveryPayload], Coroutine[Any, Any, None]]
+        | None = None,
     ) -> None:
         """Initialize the discovery update mixin."""
         self._discovery_data = discovery_data
         self._discovery_update = discovery_update
-        self._remove_discovery_updated: Callable | None = None
+        self._remove_discovery_updated: Callable[[], None] | None = None
         self._removed_from_hass = False
         if discovery_data is None:
             return
@@ -823,15 +811,18 @@ class MqttDiscoveryUpdate(Entity):
         """Subscribe to discovery updates."""
         await super().async_added_to_hass()
         self._removed_from_hass = False
-        discovery_hash = (
+        discovery_hash: tuple[str, str] | None = (
             self._discovery_data[ATTR_DISCOVERY_HASH] if self._discovery_data else None
         )
 
-        async def _async_remove_state_and_registry_entry(self) -> None:
+        async def _async_remove_state_and_registry_entry(
+            self: MqttDiscoveryUpdate,
+        ) -> None:
             """Remove entity's state and entity registry entry.
 
-            Remove entity from entity registry if it is registered, this also removes the state.
-            If the entity is not in the entity registry, just remove the state.
+            Remove entity from entity registry if it is registered,
+            this also removes the state. If the entity is not in the entity
+            registry, just remove the state.
             """
             entity_registry = er.async_get(self.hass)
             if entity_entry := entity_registry.async_get(self.entity_id):
@@ -842,13 +833,15 @@ class MqttDiscoveryUpdate(Entity):
             else:
                 await self.async_remove(force_remove=True)
 
-        async def discovery_callback(payload):
+        async def discovery_callback(payload: MQTTDiscoveryPayload) -> None:
             """Handle discovery update."""
-            _LOGGER.info(
+            _LOGGER.debug(
                 "Got update for entity with hash: %s '%s'",
                 discovery_hash,
                 payload,
             )
+            assert self._discovery_data
+            old_payload: DiscoveryInfoType
             old_payload = self._discovery_data[ATTR_DISCOVERY_PAYLOAD]
             debug_info.update_entity_discovery_data(self.hass, payload, self.entity_id)
             if not payload:
@@ -856,22 +849,27 @@ class MqttDiscoveryUpdate(Entity):
                 _LOGGER.info("Removing component: %s", self.entity_id)
                 self._cleanup_discovery_on_remove()
                 await _async_remove_state_and_registry_entry(self)
+                send_discovery_done(self.hass, self._discovery_data)
             elif self._discovery_update:
                 if old_payload != self._discovery_data[ATTR_DISCOVERY_PAYLOAD]:
                     # Non-empty, changed payload: Notify component
                     _LOGGER.info("Updating component: %s", self.entity_id)
-                    await self._discovery_update(payload)
+                    try:
+                        await self._discovery_update(payload)
+                    finally:
+                        send_discovery_done(self.hass, self._discovery_data)
                 else:
                     # Non-empty, unchanged payload: Ignore to avoid changing states
-                    _LOGGER.info("Ignoring unchanged update for: %s", self.entity_id)
-            send_discovery_done(self.hass, self._discovery_data)
+                    _LOGGER.debug("Ignoring unchanged update for: %s", self.entity_id)
+                    send_discovery_done(self.hass, self._discovery_data)
 
         if discovery_hash:
             assert self._discovery_data is not None
             debug_info.add_entity_discovery_data(
                 self.hass, self._discovery_data, self.entity_id
             )
-            # Set in case the entity has been removed and is re-added, for example when changing entity_id
+            # Set in case the entity has been removed and is re-added,
+            # for example when changing entity_id
             set_discovery_hash(self.hass, discovery_hash)
             self._remove_discovery_updated = async_dispatcher_connect(
                 self.hass,
@@ -882,11 +880,12 @@ class MqttDiscoveryUpdate(Entity):
     async def async_removed_from_registry(self) -> None:
         """Clear retained discovery topic in broker."""
         if not self._removed_from_hass and self._discovery_data is not None:
-            # Stop subscribing to discovery updates to not trigger when we clear the
-            # discovery topic
+            # Stop subscribing to discovery updates to not trigger when we
+            # clear the discovery topic
             self._cleanup_discovery_on_remove()
 
-            # Clear the discovery topic so the entity is not rediscovered after a restart
+            # Clear the discovery topic so the entity is not
+            # rediscovered after a restart
             await async_remove_discovery_payload(self.hass, self._discovery_data)
 
     @callback
@@ -923,39 +922,43 @@ class MqttDiscoveryUpdate(Entity):
             self._removed_from_hass = True
 
 
-def device_info_from_config(config) -> DeviceInfo | None:
+def device_info_from_specifications(
+    specifications: dict[str, Any] | None
+) -> DeviceInfo | None:
     """Return a device description for device registry."""
-    if not config:
+    if not specifications:
         return None
 
     info = DeviceInfo(
-        identifiers={(DOMAIN, id_) for id_ in config[CONF_IDENTIFIERS]},
-        connections={(conn_[0], conn_[1]) for conn_ in config[CONF_CONNECTIONS]},
+        identifiers={(DOMAIN, id_) for id_ in specifications[CONF_IDENTIFIERS]},
+        connections={
+            (conn_[0], conn_[1]) for conn_ in specifications[CONF_CONNECTIONS]
+        },
     )
 
-    if CONF_MANUFACTURER in config:
-        info[ATTR_MANUFACTURER] = config[CONF_MANUFACTURER]
+    if CONF_MANUFACTURER in specifications:
+        info[ATTR_MANUFACTURER] = specifications[CONF_MANUFACTURER]
 
-    if CONF_MODEL in config:
-        info[ATTR_MODEL] = config[CONF_MODEL]
+    if CONF_MODEL in specifications:
+        info[ATTR_MODEL] = specifications[CONF_MODEL]
 
-    if CONF_NAME in config:
-        info[ATTR_NAME] = config[CONF_NAME]
+    if CONF_NAME in specifications:
+        info[ATTR_NAME] = specifications[CONF_NAME]
 
-    if CONF_HW_VERSION in config:
-        info[ATTR_HW_VERSION] = config[CONF_HW_VERSION]
+    if CONF_HW_VERSION in specifications:
+        info[ATTR_HW_VERSION] = specifications[CONF_HW_VERSION]
 
-    if CONF_SW_VERSION in config:
-        info[ATTR_SW_VERSION] = config[CONF_SW_VERSION]
+    if CONF_SW_VERSION in specifications:
+        info[ATTR_SW_VERSION] = specifications[CONF_SW_VERSION]
 
-    if CONF_VIA_DEVICE in config:
-        info[ATTR_VIA_DEVICE] = (DOMAIN, config[CONF_VIA_DEVICE])
+    if CONF_VIA_DEVICE in specifications:
+        info[ATTR_VIA_DEVICE] = (DOMAIN, specifications[CONF_VIA_DEVICE])
 
-    if CONF_SUGGESTED_AREA in config:
-        info[ATTR_SUGGESTED_AREA] = config[CONF_SUGGESTED_AREA]
+    if CONF_SUGGESTED_AREA in specifications:
+        info[ATTR_SUGGESTED_AREA] = specifications[CONF_SUGGESTED_AREA]
 
-    if CONF_CONFIGURATION_URL in config:
-        info[ATTR_CONFIGURATION_URL] = config[CONF_CONFIGURATION_URL]
+    if CONF_CONFIGURATION_URL in specifications:
+        info[ATTR_CONFIGURATION_URL] = specifications[CONF_CONFIGURATION_URL]
 
     return info
 
@@ -963,19 +966,21 @@ def device_info_from_config(config) -> DeviceInfo | None:
 class MqttEntityDeviceInfo(Entity):
     """Mixin used for mqtt platforms that support the device registry."""
 
-    def __init__(self, device_config: ConfigType | None, config_entry=None) -> None:
+    def __init__(
+        self, specifications: dict[str, Any] | None, config_entry: ConfigEntry
+    ) -> None:
         """Initialize the device mixin."""
-        self._device_config = device_config
+        self._device_specifications = specifications
         self._config_entry = config_entry
 
-    def device_info_discovery_update(self, config: dict):
+    def device_info_discovery_update(self, config: DiscoveryInfoType) -> None:
         """Handle updated discovery message."""
-        self._device_config = config.get(CONF_DEVICE)
+        self._device_specifications = config.get(CONF_DEVICE)
         device_registry = dr.async_get(self.hass)
         config_entry_id = self._config_entry.entry_id
         device_info = self.device_info
 
-        if config_entry_id is not None and device_info is not None:
+        if device_info is not None:
             device_registry.async_get_or_create(
                 config_entry_id=config_entry_id, **device_info
             )
@@ -983,7 +988,7 @@ class MqttEntityDeviceInfo(Entity):
     @property
     def device_info(self) -> DeviceInfo | None:
         """Return a device description for device registry."""
-        return device_info_from_config(self._device_config)
+        return device_info_from_specifications(self._device_specifications)
 
 
 class MqttEntity(
@@ -997,14 +1002,21 @@ class MqttEntity(
     _attr_should_poll = False
     _entity_id_format: str
 
-    def __init__(self, hass, config, config_entry, discovery_data):
+    def __init__(
+        self,
+        hass: HomeAssistant,
+        config: ConfigType,
+        config_entry: ConfigEntry,
+        discovery_data: DiscoveryInfoType | None,
+    ) -> None:
         """Init the MQTT Entity."""
         self.hass = hass
-        self._config = config
-        self._unique_id = config.get(CONF_UNIQUE_ID)
-        self._sub_state = None
+        self._config: ConfigType = config
+        self._attr_unique_id = config.get(CONF_UNIQUE_ID)
+        self._sub_state: dict[str, EntitySubscription] = {}
 
         # Load config
+        self._setup_common_attributes_from_config(self._config)
         self._setup_from_config(self._config)
 
         # Initialize entity_id from config
@@ -1016,14 +1028,14 @@ class MqttEntity(
         MqttDiscoveryUpdate.__init__(self, hass, discovery_data, self.discovery_update)
         MqttEntityDeviceInfo.__init__(self, config.get(CONF_DEVICE), config_entry)
 
-    def _init_entity_id(self):
+    def _init_entity_id(self) -> None:
         """Set entity_id from object_id if defined in config."""
         init_entity_id_from_config(
             self.hass, self, self._config, self._entity_id_format
         )
 
     @final
-    async def async_added_to_hass(self):
+    async def async_added_to_hass(self) -> None:
         """Subscribe to MQTT events."""
         await super().async_added_to_hass()
         self._prepare_subscribe_topics()
@@ -1032,16 +1044,17 @@ class MqttEntity(
         if self._discovery_data is not None:
             send_discovery_done(self.hass, self._discovery_data)
 
-    async def mqtt_async_added_to_hass(self):
+    async def mqtt_async_added_to_hass(self) -> None:
         """Call before the discovery message is acknowledged.
 
         To be extended by subclasses.
         """
 
-    async def discovery_update(self, discovery_payload):
+    async def discovery_update(self, discovery_payload: MQTTDiscoveryPayload) -> None:
         """Handle updated discovery message."""
-        config = self.config_schema()(discovery_payload)
+        config: DiscoveryInfoType = self.config_schema()(discovery_payload)
         self._config = config
+        self._setup_common_attributes_from_config(self._config)
         self._setup_from_config(self._config)
 
         # Prepare MQTT subscriptions
@@ -1056,7 +1069,7 @@ class MqttEntity(
         await self._subscribe_topics()
         self.async_write_ha_state()
 
-    async def async_will_remove_from_hass(self):
+    async def async_will_remove_from_hass(self) -> None:
         """Unsubscribe when removed."""
         self._sub_state = subscription.async_unsubscribe_topics(
             self.hass, self._sub_state
@@ -1072,8 +1085,8 @@ class MqttEntity(
         payload: PublishPayloadType,
         qos: int = 0,
         retain: bool = False,
-        encoding: str = DEFAULT_ENCODING,
-    ):
+        encoding: str | None = DEFAULT_ENCODING,
+    ) -> None:
         """Publish message to an MQTT topic."""
         log_message(self.hass, self.entity_id, topic, payload, qos, retain)
         await async_publish(
@@ -1087,45 +1100,29 @@ class MqttEntity(
 
     @staticmethod
     @abstractmethod
-    def config_schema():
+    def config_schema() -> vol.Schema:
         """Return the config schema."""
 
-    def _setup_from_config(self, config):
+    def _setup_common_attributes_from_config(self, config: ConfigType) -> None:
+        """(Re)Setup the common attributes for the entity."""
+        self._attr_entity_category = config.get(CONF_ENTITY_CATEGORY)
+        self._attr_entity_registry_enabled_default = bool(
+            config.get(CONF_ENABLED_BY_DEFAULT)
+        )
+        self._attr_icon = config.get(CONF_ICON)
+        self._attr_name = config.get(CONF_NAME)
+
+    def _setup_from_config(self, config: ConfigType) -> None:
         """(Re)Setup the entity."""
 
     @abstractmethod
-    def _prepare_subscribe_topics(self):
+    def _prepare_subscribe_topics(self) -> None:
         """(Re)Subscribe to topics."""
 
     @abstractmethod
-    async def _subscribe_topics(self):
+    async def _subscribe_topics(self) -> None:
         """(Re)Subscribe to topics."""
 
-    @property
-    def entity_registry_enabled_default(self) -> bool:
-        """Return if the entity should be enabled when first added to the entity registry."""
-        return self._config[CONF_ENABLED_BY_DEFAULT]
-
-    @property
-    def entity_category(self) -> EntityCategory | None:
-        """Return the entity category if any."""
-        return self._config.get(CONF_ENTITY_CATEGORY)
-
-    @property
-    def icon(self):
-        """Return icon of the entity if any."""
-        return self._config.get(CONF_ICON)
-
-    @property
-    def name(self):
-        """Return the name of the device if any."""
-        return self._config.get(CONF_NAME)
-
-    @property
-    def unique_id(self):
-        """Return a unique ID."""
-        return self._unique_id
-
 
 def update_device(
     hass: HomeAssistant,
@@ -1136,13 +1133,13 @@ def update_device(
     if CONF_DEVICE not in config:
         return None
 
-    device = None
+    device: DeviceEntry | None = None
     device_registry = dr.async_get(hass)
     config_entry_id = config_entry.entry_id
-    device_info = device_info_from_config(config[CONF_DEVICE])
+    device_info = device_info_from_specifications(config[CONF_DEVICE])
 
     if config_entry_id is not None and device_info is not None:
-        update_device_info = cast(dict, device_info)
+        update_device_info = cast(dict[str, Any], device_info)
         update_device_info["config_entry_id"] = config_entry_id
         device = device_registry.async_get_or_create(**update_device_info)
 
@@ -1154,7 +1151,7 @@ def async_removed_from_device(
     hass: HomeAssistant, event: Event, mqtt_device_id: str, config_entry_id: str
 ) -> bool:
     """Check if the passed event indicates MQTT was removed from a device."""
-    device_id = event.data["device_id"]
+    device_id: str = event.data["device_id"]
     if event.data["action"] not in ("remove", "update"):
         return False
 
