@@ -1,8 +1,7 @@
 """Connect to a MySensors gateway via pymysensors API."""
 from __future__ import annotations
 
-from collections.abc import Callable
-from functools import partial
+from collections.abc import Callable, Mapping
 import logging
 
 from mysensors import BaseAsyncGateway
@@ -12,24 +11,19 @@ from homeassistant.const import Platform
 from homeassistant.core import HomeAssistant, callback
 import homeassistant.helpers.config_validation as cv
 from homeassistant.helpers.device_registry import DeviceEntry
-from homeassistant.helpers.discovery import async_load_platform
-from homeassistant.helpers.dispatcher import async_dispatcher_connect
-from homeassistant.helpers.typing import ConfigType
 
 from .const import (
     ATTR_DEVICES,
     DOMAIN,
-    MYSENSORS_DISCOVERY,
     MYSENSORS_GATEWAYS,
     MYSENSORS_ON_UNLOAD,
-    PLATFORMS_WITH_ENTRY_SUPPORT,
+    PLATFORMS,
     DevId,
     DiscoveryInfo,
     SensorType,
 )
-from .device import MySensorsDevice, get_mysensors_devices
+from .device import MySensorsEntity, get_mysensors_devices
 from .gateway import finish_setup, gw_stop, setup_gateway
-from .helpers import on_unload
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -39,14 +33,6 @@ DATA_HASS_CONFIG = "hass_config"
 CONFIG_SCHEMA = cv.removed(DOMAIN, raise_if_present=False)
 
 
-async def async_setup(hass: HomeAssistant, config: ConfigType) -> bool:
-    """Set up the MySensors component."""
-    # This is needed to set up the notify platform via discovery.
-    hass.data[DOMAIN] = {DATA_HASS_CONFIG: config}
-
-    return True
-
-
 async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
     """Set up an instance of the MySensors integration.
 
@@ -58,36 +44,12 @@ async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
         _LOGGER.error("Gateway setup failed for %s", entry.data)
         return False
 
-    if MYSENSORS_GATEWAYS not in hass.data[DOMAIN]:
-        hass.data[DOMAIN][MYSENSORS_GATEWAYS] = {}
-    hass.data[DOMAIN][MYSENSORS_GATEWAYS][entry.entry_id] = gateway
+    mysensors_data = hass.data.setdefault(DOMAIN, {})
+    if MYSENSORS_GATEWAYS not in mysensors_data:
+        mysensors_data[MYSENSORS_GATEWAYS] = {}
+    mysensors_data[MYSENSORS_GATEWAYS][entry.entry_id] = gateway
 
-    # Connect notify discovery as that integration doesn't support entry forwarding.
-    # Allow loading device tracker platform via discovery
-    # until refactor to config entry is done.
-
-    for platform in (Platform.DEVICE_TRACKER, Platform.NOTIFY):
-        load_discovery_platform = partial(
-            async_load_platform,
-            hass,
-            platform,
-            DOMAIN,
-            hass_config=hass.data[DOMAIN][DATA_HASS_CONFIG],
-        )
-
-        on_unload(
-            hass,
-            entry.entry_id,
-            async_dispatcher_connect(
-                hass,
-                MYSENSORS_DISCOVERY.format(entry.entry_id, platform),
-                load_discovery_platform,
-            ),
-        )
-
-    await hass.config_entries.async_forward_entry_setups(
-        entry, PLATFORMS_WITH_ENTRY_SUPPORT
-    )
+    await hass.config_entries.async_forward_entry_setups(entry, PLATFORMS)
     await finish_setup(hass, entry, gateway)
 
     return True
@@ -98,9 +60,7 @@ async def async_unload_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
 
     gateway: BaseAsyncGateway = hass.data[DOMAIN][MYSENSORS_GATEWAYS][entry.entry_id]
 
-    unload_ok = await hass.config_entries.async_unload_platforms(
-        entry, PLATFORMS_WITH_ENTRY_SUPPORT
-    )
+    unload_ok = await hass.config_entries.async_unload_platforms(entry, PLATFORMS)
     if not unload_ok:
         return False
 
@@ -139,25 +99,29 @@ def setup_mysensors_platform(
     hass: HomeAssistant,
     domain: Platform,  # hass platform name
     discovery_info: DiscoveryInfo,
-    device_class: type[MySensorsDevice] | dict[SensorType, type[MySensorsDevice]],
+    device_class: type[MySensorsEntity] | Mapping[SensorType, type[MySensorsEntity]],
     device_args: (
         None | tuple
     ) = None,  # extra arguments that will be given to the entity constructor
     async_add_entities: Callable | None = None,
-) -> list[MySensorsDevice] | None:
+) -> list[MySensorsEntity] | None:
     """Set up a MySensors platform.
 
-    Sets up a bunch of instances of a single platform that is supported by this integration.
-    The function is given a list of device ids, each one describing an instance to set up.
-    The function is also given a class.
-    A new instance of the class is created for every device id, and the device id is given to the constructor of the class
+    Sets up a bunch of instances of a single platform that is supported by this
+    integration.
+
+    The function is given a list of device ids, each one describing an instance
+    to set up. The function is also given a class.
+
+    A new instance of the class is created for every device id, and the device
+    id is given to the constructor of the class.
     """
     if device_args is None:
         device_args = ()
-    new_devices: list[MySensorsDevice] = []
+    new_devices: list[MySensorsEntity] = []
     new_dev_ids: list[DevId] = discovery_info[ATTR_DEVICES]
     for dev_id in new_dev_ids:
-        devices: dict[DevId, MySensorsDevice] = get_mysensors_devices(hass, domain)
+        devices: dict[DevId, MySensorsEntity] = get_mysensors_devices(hass, domain)
         if dev_id in devices:
             _LOGGER.debug(
                 "Skipping setup of %s for platform %s as it already exists",
