@@ -1,56 +1,56 @@
 """Support for MySensors lights."""
-from homeassistant.components import mysensors
+from __future__ import annotations
+
+from typing import Any, cast
+
 from homeassistant.components.light import (
     ATTR_BRIGHTNESS,
-    ATTR_HS_COLOR,
-    ATTR_WHITE_VALUE,
-    DOMAIN,
-    SUPPORT_BRIGHTNESS,
-    SUPPORT_COLOR,
-    SUPPORT_WHITE_VALUE,
+    ATTR_RGB_COLOR,
+    ATTR_RGBW_COLOR,
+    ColorMode,
     LightEntity,
 )
-from homeassistant.components.mysensors import on_unload
-from homeassistant.components.mysensors.const import MYSENSORS_DISCOVERY
 from homeassistant.config_entries import ConfigEntry
-from homeassistant.const import STATE_OFF, STATE_ON
+from homeassistant.const import STATE_OFF, STATE_ON, Platform
 from homeassistant.core import HomeAssistant, callback
 from homeassistant.helpers.dispatcher import async_dispatcher_connect
 from homeassistant.helpers.entity_platform import AddEntitiesCallback
-import homeassistant.util.color as color_util
 from homeassistant.util.color import rgb_hex_to_rgb_list
 
-SUPPORT_MYSENSORS_RGBW = SUPPORT_COLOR | SUPPORT_WHITE_VALUE
+from .. import mysensors
+from .const import MYSENSORS_DISCOVERY, DiscoveryInfo, SensorType
+from .device import MySensorsEntity
+from .helpers import on_unload
 
 
 async def async_setup_entry(
     hass: HomeAssistant,
     config_entry: ConfigEntry,
     async_add_entities: AddEntitiesCallback,
-):
+) -> None:
     """Set up this platform for a specific ConfigEntry(==Gateway)."""
-    device_class_map = {
+    device_class_map: dict[SensorType, type[MySensorsEntity]] = {
         "S_DIMMER": MySensorsLightDimmer,
         "S_RGB_LIGHT": MySensorsLightRGB,
         "S_RGBW_LIGHT": MySensorsLightRGBW,
     }
 
-    async def async_discover(discovery_info):
+    async def async_discover(discovery_info: DiscoveryInfo) -> None:
         """Discover and add a MySensors light."""
         mysensors.setup_mysensors_platform(
             hass,
-            DOMAIN,
+            Platform.LIGHT,
             discovery_info,
             device_class_map,
             async_add_entities=async_add_entities,
         )
 
-    await on_unload(
+    on_unload(
         hass,
-        config_entry,
+        config_entry.entry_id,
         async_dispatcher_connect(
             hass,
-            MYSENSORS_DISCOVERY.format(config_entry.entry_id, DOMAIN),
+            MYSENSORS_DISCOVERY.format(config_entry.entry_id, Platform.LIGHT),
             async_discover,
         ),
     )
@@ -59,35 +59,17 @@ async def async_setup_entry(
 class MySensorsLight(mysensors.device.MySensorsEntity, LightEntity):
     """Representation of a MySensors Light child node."""
 
-    def __init__(self, *args):
+    def __init__(self, *args: Any) -> None:
         """Initialize a MySensors Light."""
         super().__init__(*args)
-        self._state = None
-        self._brightness = None
-        self._hs = None
-        self._white = None
+        self._state: bool | None = None
 
     @property
-    def brightness(self):
-        """Return the brightness of this light between 0..255."""
-        return self._brightness
-
-    @property
-    def hs_color(self):
-        """Return the hs color value [int, int]."""
-        return self._hs
-
-    @property
-    def white_value(self):
-        """Return the white value of this light between 0..255."""
-        return self._white
-
-    @property
-    def is_on(self):
+    def is_on(self) -> bool:
         """Return true if device is on."""
-        return self._state
+        return bool(self._state)
 
-    def _turn_on_light(self):
+    def _turn_on_light(self) -> None:
         """Turn on light child device."""
         set_req = self.gateway.const.SetReq
 
@@ -102,18 +84,17 @@ class MySensorsLight(mysensors.device.MySensorsEntity, LightEntity):
             self._state = True
             self._values[set_req.V_LIGHT] = STATE_ON
 
-    def _turn_on_dimmer(self, **kwargs):
+    def _turn_on_dimmer(self, **kwargs: Any) -> None:
         """Turn on dimmer child device."""
         set_req = self.gateway.const.SetReq
-        brightness = self._brightness
 
         if (
             ATTR_BRIGHTNESS not in kwargs
-            or kwargs[ATTR_BRIGHTNESS] == self._brightness
+            or kwargs[ATTR_BRIGHTNESS] == self._attr_brightness
             or set_req.V_DIMMER not in self._values
         ):
             return
-        brightness = kwargs[ATTR_BRIGHTNESS]
+        brightness: int = kwargs[ATTR_BRIGHTNESS]
         percent = round(100 * brightness / 255)
         self.gateway.set_child_value(
             self.node_id, self.child_id, set_req.V_DIMMER, percent, ack=1
@@ -121,44 +102,10 @@ class MySensorsLight(mysensors.device.MySensorsEntity, LightEntity):
 
         if self.assumed_state:
             # optimistically assume that light has changed state
-            self._brightness = brightness
+            self._attr_brightness = brightness
             self._values[set_req.V_DIMMER] = percent
 
-    def _turn_on_rgb_and_w(self, hex_template, **kwargs):
-        """Turn on RGB or RGBW child device."""
-        rgb = list(color_util.color_hs_to_RGB(*self._hs))
-        white = self._white
-        hex_color = self._values.get(self.value_type)
-        hs_color = kwargs.get(ATTR_HS_COLOR)
-        if hs_color is not None:
-            new_rgb = color_util.color_hs_to_RGB(*hs_color)
-        else:
-            new_rgb = None
-        new_white = kwargs.get(ATTR_WHITE_VALUE)
-
-        if new_rgb is None and new_white is None:
-            return
-        if new_rgb is not None:
-            rgb = list(new_rgb)
-        if hex_template == "%02x%02x%02x%02x":
-            if new_white is not None:
-                rgb.append(new_white)
-            else:
-                rgb.append(white)
-        hex_color = hex_template % tuple(rgb)
-        if len(rgb) > 3:
-            white = rgb.pop()
-        self.gateway.set_child_value(
-            self.node_id, self.child_id, self.value_type, hex_color, ack=1
-        )
-
-        if self.assumed_state:
-            # optimistically assume that light has changed state
-            self._hs = color_util.color_RGB_to_hs(*rgb)
-            self._white = white
-            self._values[self.value_type] = hex_color
-
-    async def async_turn_off(self, **kwargs):
+    async def async_turn_off(self, **kwargs: Any) -> None:
         """Turn the device off."""
         value_type = self.gateway.const.SetReq.V_LIGHT
         self.gateway.set_child_value(self.node_id, self.child_id, value_type, 0, ack=1)
@@ -169,48 +116,38 @@ class MySensorsLight(mysensors.device.MySensorsEntity, LightEntity):
             self.async_write_ha_state()
 
     @callback
-    def _async_update_light(self):
+    def _async_update_light(self) -> None:
         """Update the controller with values from light child."""
         value_type = self.gateway.const.SetReq.V_LIGHT
         self._state = self._values[value_type] == STATE_ON
 
     @callback
-    def _async_update_dimmer(self):
+    def _async_update_dimmer(self) -> None:
         """Update the controller with values from dimmer child."""
         value_type = self.gateway.const.SetReq.V_DIMMER
         if value_type in self._values:
-            self._brightness = round(255 * int(self._values[value_type]) / 100)
-            if self._brightness == 0:
+            self._attr_brightness = round(255 * int(self._values[value_type]) / 100)
+            if self._attr_brightness == 0:
                 self._state = False
 
-    @callback
-    def _async_update_rgb_or_w(self):
-        """Update the controller with values from RGB or RGBW child."""
-        value = self._values[self.value_type]
-        color_list = rgb_hex_to_rgb_list(value)
-        if len(color_list) > 3:
-            self._white = color_list.pop()
-        self._hs = color_util.color_RGB_to_hs(*color_list)
-
 
 class MySensorsLightDimmer(MySensorsLight):
     """Dimmer child class to MySensorsLight."""
 
-    @property
-    def supported_features(self):
-        """Flag supported features."""
-        return SUPPORT_BRIGHTNESS
+    _attr_supported_color_modes = {ColorMode.BRIGHTNESS}
+    _attr_color_mode = ColorMode.BRIGHTNESS
 
-    async def async_turn_on(self, **kwargs):
+    async def async_turn_on(self, **kwargs: Any) -> None:
         """Turn the device on."""
         self._turn_on_light()
         self._turn_on_dimmer(**kwargs)
         if self.assumed_state:
             self.async_write_ha_state()
 
-    async def async_update(self):
+    @callback
+    def _async_update(self) -> None:
         """Update the controller with the latest value from a sensor."""
-        await super().async_update()
+        super()._async_update()
         self._async_update_light()
         self._async_update_dimmer()
 
@@ -218,45 +155,84 @@ class MySensorsLightDimmer(MySensorsLight):
 class MySensorsLightRGB(MySensorsLight):
     """RGB child class to MySensorsLight."""
 
-    @property
-    def supported_features(self):
-        """Flag supported features."""
-        set_req = self.gateway.const.SetReq
-        if set_req.V_DIMMER in self._values:
-            return SUPPORT_BRIGHTNESS | SUPPORT_COLOR
-        return SUPPORT_COLOR
+    _attr_supported_color_modes = {ColorMode.RGB}
+    _attr_color_mode = ColorMode.RGB
 
-    async def async_turn_on(self, **kwargs):
+    async def async_turn_on(self, **kwargs: Any) -> None:
         """Turn the device on."""
         self._turn_on_light()
         self._turn_on_dimmer(**kwargs)
-        self._turn_on_rgb_and_w("%02x%02x%02x", **kwargs)
+        self._turn_on_rgb(**kwargs)
         if self.assumed_state:
             self.async_write_ha_state()
 
-    async def async_update(self):
+    def _turn_on_rgb(self, **kwargs: Any) -> None:
+        """Turn on RGB child device."""
+        hex_color = self._values.get(self.value_type)
+        new_rgb: tuple[int, int, int] | None = kwargs.get(ATTR_RGB_COLOR)
+        if new_rgb is None:
+            return
+        hex_color = "%02x%02x%02x" % new_rgb
+        self.gateway.set_child_value(
+            self.node_id, self.child_id, self.value_type, hex_color, ack=1
+        )
+
+        if self.assumed_state:
+            # optimistically assume that light has changed state
+            self._attr_rgb_color = new_rgb
+            self._values[self.value_type] = hex_color
+
+    @callback
+    def _async_update(self) -> None:
         """Update the controller with the latest value from a sensor."""
-        await super().async_update()
+        super()._async_update()
         self._async_update_light()
         self._async_update_dimmer()
         self._async_update_rgb_or_w()
 
+    @callback
+    def _async_update_rgb_or_w(self) -> None:
+        """Update the controller with values from RGB child."""
+        value = self._values[self.value_type]
+        self._attr_rgb_color = cast(
+            tuple[int, int, int], tuple(rgb_hex_to_rgb_list(value))
+        )
+
 
 class MySensorsLightRGBW(MySensorsLightRGB):
     """RGBW child class to MySensorsLightRGB."""
 
-    @property
-    def supported_features(self):
-        """Flag supported features."""
-        set_req = self.gateway.const.SetReq
-        if set_req.V_DIMMER in self._values:
-            return SUPPORT_BRIGHTNESS | SUPPORT_MYSENSORS_RGBW
-        return SUPPORT_MYSENSORS_RGBW
+    _attr_supported_color_modes = {ColorMode.RGBW}
+    _attr_color_mode = ColorMode.RGBW
 
-    async def async_turn_on(self, **kwargs):
+    async def async_turn_on(self, **kwargs: Any) -> None:
         """Turn the device on."""
         self._turn_on_light()
         self._turn_on_dimmer(**kwargs)
-        self._turn_on_rgb_and_w("%02x%02x%02x%02x", **kwargs)
+        self._turn_on_rgbw(**kwargs)
         if self.assumed_state:
             self.async_write_ha_state()
+
+    def _turn_on_rgbw(self, **kwargs: Any) -> None:
+        """Turn on RGBW child device."""
+        hex_color = self._values.get(self.value_type)
+        new_rgbw: tuple[int, int, int, int] | None = kwargs.get(ATTR_RGBW_COLOR)
+        if new_rgbw is None:
+            return
+        hex_color = "%02x%02x%02x%02x" % new_rgbw
+        self.gateway.set_child_value(
+            self.node_id, self.child_id, self.value_type, hex_color, ack=1
+        )
+
+        if self.assumed_state:
+            # optimistically assume that light has changed state
+            self._attr_rgbw_color = new_rgbw
+            self._values[self.value_type] = hex_color
+
+    @callback
+    def _async_update_rgb_or_w(self) -> None:
+        """Update the controller with values from RGBW child."""
+        value = self._values[self.value_type]
+        self._attr_rgbw_color = cast(
+            tuple[int, int, int, int], tuple(rgb_hex_to_rgb_list(value))
+        )
