@@ -1,17 +1,24 @@
 """The Panasonic Viera integration."""
 from functools import partial
 import logging
-from urllib.request import URLError
+from urllib.error import HTTPError, URLError
 
 from panasonic_viera import EncryptionRequired, Keys, RemoteControl, SOAPError
 import voluptuous as vol
 
-from homeassistant.components.media_player.const import DOMAIN as MEDIA_PLAYER_DOMAIN
-from homeassistant.components.remote import DOMAIN as REMOTE_DOMAIN
-from homeassistant.config_entries import SOURCE_IMPORT
-from homeassistant.const import CONF_HOST, CONF_NAME, CONF_PORT, STATE_OFF, STATE_ON
+from homeassistant.config_entries import SOURCE_IMPORT, ConfigEntry
+from homeassistant.const import (
+    CONF_HOST,
+    CONF_NAME,
+    CONF_PORT,
+    STATE_OFF,
+    STATE_ON,
+    Platform,
+)
+from homeassistant.core import HomeAssistant
 import homeassistant.helpers.config_validation as cv
 from homeassistant.helpers.script import Script
+from homeassistant.helpers.typing import ConfigType
 
 from .const import (
     ATTR_DEVICE_INFO,
@@ -46,10 +53,10 @@ CONFIG_SCHEMA = vol.Schema(
     extra=vol.ALLOW_EXTRA,
 )
 
-PLATFORMS = [MEDIA_PLAYER_DOMAIN, REMOTE_DOMAIN]
+PLATFORMS = [Platform.MEDIA_PLAYER, Platform.REMOTE]
 
 
-async def async_setup(hass, config):
+async def async_setup(hass: HomeAssistant, config: ConfigType) -> bool:
     """Set up Panasonic Viera from configuration.yaml."""
     if DOMAIN not in config:
         return True
@@ -64,7 +71,7 @@ async def async_setup(hass, config):
     return True
 
 
-async def async_setup_entry(hass, config_entry):
+async def async_setup_entry(hass: HomeAssistant, config_entry: ConfigEntry) -> bool:
     """Set up Panasonic Viera from a config entry."""
     panasonic_viera_data = hass.data.setdefault(DOMAIN, {})
 
@@ -73,8 +80,7 @@ async def async_setup_entry(hass, config_entry):
     host = config[CONF_HOST]
     port = config[CONF_PORT]
 
-    on_action = config[CONF_ON_ACTION]
-    if on_action is not None:
+    if (on_action := config[CONF_ON_ACTION]) is not None:
         on_action = Script(hass, on_action, config[CONF_NAME], DOMAIN)
 
     params = {}
@@ -93,7 +99,8 @@ async def async_setup_entry(hass, config_entry):
         unique_id = config_entry.unique_id
         if device_info is None:
             _LOGGER.error(
-                "Couldn't gather device info; Please restart Home Assistant with your TV turned on and connected to your network"
+                "Couldn't gather device info; Please restart Home Assistant with your"
+                " TV turned on and connected to your network"
             )
         else:
             unique_id = device_info[ATTR_UDN]
@@ -103,12 +110,12 @@ async def async_setup_entry(hass, config_entry):
             data={**config, ATTR_DEVICE_INFO: device_info},
         )
 
-    hass.config_entries.async_setup_platforms(config_entry, PLATFORMS)
+    await hass.config_entries.async_forward_entry_setups(config_entry, PLATFORMS)
 
     return True
 
 
-async def async_unload_entry(hass, config_entry):
+async def async_unload_entry(hass: HomeAssistant, config_entry: ConfigEntry) -> bool:
     """Unload a config entry."""
     unload_ok = await hass.config_entries.async_unload_platforms(
         config_entry, PLATFORMS
@@ -247,11 +254,13 @@ class Remote:
                 "The connection couldn't be encrypted. Please reconfigure your TV"
             )
             self.available = False
-        except (SOAPError):
+        except (SOAPError, HTTPError) as err:
+            _LOGGER.debug("An error occurred: %s", err)
             self.state = STATE_OFF
             self.available = True
             await self.async_create_remote_control()
-        except (URLError, OSError):
+        except (URLError, OSError) as err:
+            _LOGGER.debug("An error occurred: %s", err)
             self.state = STATE_OFF
             self.available = self._on_action is not None
             await self.async_create_remote_control()
