@@ -3,11 +3,11 @@ from __future__ import annotations
 
 from collections.abc import Callable
 import logging
-from typing import TYPE_CHECKING, Any, TypeVar
+from typing import TYPE_CHECKING, Any, Concatenate, ParamSpec, TypeVar, overload
 
+from requests.exceptions import Timeout
 from soco import SoCo
 from soco.exceptions import SoCoException, SoCoUPnPException
-from typing_extensions import Concatenate, ParamSpec
 
 from homeassistant.helpers.dispatcher import dispatcher_send
 
@@ -17,6 +17,7 @@ from .exception import SonosUpdateError
 if TYPE_CHECKING:
     from .entity import SonosEntity
     from .household_coordinator import SonosHouseholdCoordinator
+    from .media import SonosMedia
     from .speaker import SonosSpeaker
 
 UID_PREFIX = "RINCON_"
@@ -24,21 +25,36 @@ UID_POSTFIX = "01400"
 
 _LOGGER = logging.getLogger(__name__)
 
-_T = TypeVar("_T", bound="SonosSpeaker | SonosEntity | SonosHouseholdCoordinator")
+_T = TypeVar(
+    "_T", bound="SonosSpeaker | SonosMedia | SonosEntity | SonosHouseholdCoordinator"
+)
 _R = TypeVar("_R")
 _P = ParamSpec("_P")
 
+_FuncType = Callable[Concatenate[_T, _P], _R]
+_ReturnFuncType = Callable[Concatenate[_T, _P], _R | None]
+
+
+@overload
+def soco_error(
+    errorcodes: None = ...,
+) -> Callable[[_FuncType[_T, _P, _R]], _FuncType[_T, _P, _R]]:
+    ...
+
+
+@overload
+def soco_error(
+    errorcodes: list[str],
+) -> Callable[[_FuncType[_T, _P, _R]], _ReturnFuncType[_T, _P, _R]]:
+    ...
+
 
 def soco_error(
     errorcodes: list[str] | None = None,
-) -> Callable[  # type: ignore[misc]
-    [Callable[Concatenate[_T, _P], _R]], Callable[Concatenate[_T, _P], _R | None]
-]:
+) -> Callable[[_FuncType[_T, _P, _R]], _ReturnFuncType[_T, _P, _R]]:
     """Filter out specified UPnP errors and raise exceptions for service calls."""
 
-    def decorator(
-        funct: Callable[Concatenate[_T, _P], _R]  # type: ignore[misc]
-    ) -> Callable[Concatenate[_T, _P], _R | None]:  # type: ignore[misc]
+    def decorator(funct: _FuncType[_T, _P, _R]) -> _ReturnFuncType[_T, _P, _R]:
         """Decorate functions."""
 
         def wrapper(self: _T, *args: _P.args, **kwargs: _P.kwargs) -> _R | None:
@@ -46,7 +62,7 @@ def soco_error(
             args_soco = next((arg for arg in args if isinstance(arg, SoCo)), None)
             try:
                 result = funct(self, *args, **kwargs)
-            except (OSError, SoCoException, SoCoUPnPException) as err:
+            except (OSError, SoCoException, SoCoUPnPException, Timeout) as err:
                 error_code = getattr(err, "error_code", None)
                 function = funct.__qualname__
                 if errorcodes and error_code in errorcodes:
@@ -61,7 +77,7 @@ def soco_error(
                 message = f"Error calling {function} on {target}: {err}"
                 raise SonosUpdateError(message) from err
 
-            dispatch_soco = args_soco or self.soco
+            dispatch_soco = args_soco or self.soco  # type: ignore[union-attr]
             dispatcher_send(
                 self.hass,
                 f"{SONOS_SPEAKER_ACTIVITY}-{dispatch_soco.uid}",
@@ -75,7 +91,7 @@ def soco_error(
 
 
 def _find_target_identifier(instance: Any, fallback_soco: SoCo | None) -> str | None:
-    """Extract the the best available target identifier from the provided instance object."""
+    """Extract the best available target identifier from the provided instance object."""
     if entity_id := getattr(instance, "entity_id", None):
         # SonosEntity instance
         return entity_id
@@ -95,9 +111,9 @@ def _find_target_identifier(instance: Any, fallback_soco: SoCo | None) -> str |
 def hostname_to_uid(hostname: str) -> str:
     """Convert a Sonos hostname to a uid."""
     if hostname.startswith("Sonos-"):
-        baseuid = hostname.split("-")[1].replace(".local.", "")
+        baseuid = hostname.removeprefix("Sonos-").replace(".local.", "")
     elif hostname.startswith("sonos"):
-        baseuid = hostname[5:].replace(".local.", "")
+        baseuid = hostname.removeprefix("sonos").replace(".local.", "")
     else:
         raise ValueError(f"{hostname} is not a sonos device.")
     return f"{UID_PREFIX}{baseuid}{UID_POSTFIX}"
