@@ -1,25 +1,31 @@
 """Config flow for System Bridge integration."""
 from __future__ import annotations
 
+import asyncio
+from collections.abc import Mapping
 import logging
 from typing import Any
 
 import async_timeout
-from systembridge import Bridge
-from systembridge.client import BridgeClient
-from systembridge.exceptions import BridgeAuthenticationException
-from systembridge.objects.os import Os
-from systembridge.objects.system import System
+from systembridgeconnector.exceptions import (
+    AuthenticationException,
+    ConnectionClosedException,
+    ConnectionErrorException,
+)
+from systembridgeconnector.models.get_data import GetData
+from systembridgeconnector.models.system import System
+from systembridgeconnector.websocket_client import WebSocketClient
 import voluptuous as vol
 
 from homeassistant import config_entries, exceptions
+from homeassistant.components import zeroconf
 from homeassistant.const import CONF_API_KEY, CONF_HOST, CONF_PORT
 from homeassistant.core import HomeAssistant
 from homeassistant.data_entry_flow import FlowResult
-from homeassistant.helpers import aiohttp_client, config_validation as cv
-from homeassistant.helpers.typing import ConfigType, DiscoveryInfoType
+from homeassistant.helpers import config_validation as cv
+from homeassistant.helpers.aiohttp_client import async_get_clientsession
 
-from .const import BRIDGE_CONNECTION_ERRORS, DOMAIN
+from .const import DOMAIN
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -33,64 +39,89 @@ STEP_USER_DATA_SCHEMA = vol.Schema(
 )
 
 
-async def validate_input(hass: HomeAssistant, data: dict[str, Any]) -> dict[str, str]:
+async def _validate_input(
+    hass: HomeAssistant,
+    data: dict[str, Any],
+) -> dict[str, str]:
     """Validate the user input allows us to connect.
 
     Data has the keys from STEP_USER_DATA_SCHEMA with values provided by the user.
     """
-    bridge = Bridge(
-        BridgeClient(aiohttp_client.async_get_clientsession(hass)),
-        f"http://{data[CONF_HOST]}:{data[CONF_PORT]}",
+    host = data[CONF_HOST]
+
+    websocket_client = WebSocketClient(
+        host,
+        data[CONF_PORT],
         data[CONF_API_KEY],
     )
-
-    hostname = data[CONF_HOST]
     try:
         async with async_timeout.timeout(30):
-            bridge_os: Os = await bridge.async_get_os()
-            if bridge_os.hostname is not None:
-                hostname = bridge_os.hostname
-            bridge_system: System = await bridge.async_get_system()
-    except BridgeAuthenticationException as exception:
-        _LOGGER.info(exception)
+            await websocket_client.connect(session=async_get_clientsession(hass))
+            hass.async_create_task(websocket_client.listen())
+            response = await websocket_client.get_data(GetData(modules=["system"]))
+            _LOGGER.debug("Got response: %s", response.json())
+            if response.data is None or not isinstance(response.data, System):
+                raise CannotConnect("No data received")
+            system: System = response.data
+    except AuthenticationException as exception:
+        _LOGGER.warning(
+            "Authentication error when connecting to %s: %s", data[CONF_HOST], exception
+        )
         raise InvalidAuth from exception
-    except BRIDGE_CONNECTION_ERRORS as exception:
-        _LOGGER.info(exception)
+    except (
+        ConnectionClosedException,
+        ConnectionErrorException,
+    ) as exception:
+        _LOGGER.warning(
+            "Connection error when connecting to %s: %s", data[CONF_HOST], exception
+        )
+        raise CannotConnect from exception
+    except asyncio.TimeoutError as exception:
+        _LOGGER.warning("Timed out connecting to %s: %s", data[CONF_HOST], exception)
+        raise CannotConnect from exception
+    except ValueError as exception:
         raise CannotConnect from exception
 
-    return {"hostname": hostname, "uuid": bridge_system.uuid.os}
+    _LOGGER.debug("Got System data: %s", system.json())
+
+    return {"hostname": host, "uuid": system.uuid}
 
 
-class ConfigFlow(config_entries.ConfigFlow, domain=DOMAIN):
+async def _async_get_info(
+    hass: HomeAssistant,
+    user_input: dict[str, Any],
+) -> tuple[dict[str, str], dict[str, str] | None]:
+    errors = {}
+
+    try:
+        info = await _validate_input(hass, user_input)
+    except CannotConnect:
+        errors["base"] = "cannot_connect"
+    except InvalidAuth:
+        errors["base"] = "invalid_auth"
+    except Exception:  # pylint: disable=broad-except
+        _LOGGER.exception("Unexpected exception")
+        errors["base"] = "unknown"
+    else:
+        return errors, info
+
+    return errors, None
+
+
+class ConfigFlow(
+    config_entries.ConfigFlow,
+    domain=DOMAIN,
+):
     """Handle a config flow for System Bridge."""
 
     VERSION = 1
 
-    def __init__(self):
+    def __init__(self) -> None:
         """Initialize flow."""
         self._name: str | None = None
         self._input: dict[str, Any] = {}
         self._reauth = False
 
-    async def _async_get_info(
-        self, user_input: dict[str, Any]
-    ) -> tuple[dict[str, str], dict[str, str] | None]:
-        errors = {}
-
-        try:
-            info = await validate_input(self.hass, user_input)
-        except CannotConnect:
-            errors["base"] = "cannot_connect"
-        except InvalidAuth:
-            errors["base"] = "invalid_auth"
-        except Exception:  # pylint: disable=broad-except
-            _LOGGER.exception("Unexpected exception")
-            errors["base"] = "unknown"
-        else:
-            return errors, info
-
-        return errors, None
-
     async def async_step_user(
         self, user_input: dict[str, Any] | None = None
     ) -> FlowResult:
@@ -100,7 +131,7 @@ class ConfigFlow(config_entries.ConfigFlow, domain=DOMAIN):
                 step_id="user", data_schema=STEP_USER_DATA_SCHEMA
             )
 
-        errors, info = await self._async_get_info(user_input)
+        errors, info = await _async_get_info(self.hass, user_input)
         if not errors and info is not None:
             # Check if already configured
             await self.async_set_unique_id(info["uuid"], raise_on_progress=False)
@@ -120,7 +151,7 @@ class ConfigFlow(config_entries.ConfigFlow, domain=DOMAIN):
 
         if user_input is not None:
             user_input = {**self._input, **user_input}
-            errors, info = await self._async_get_info(user_input)
+            errors, info = await _async_get_info(self.hass, user_input)
             if not errors and info is not None:
                 # Check if already configured
                 existing_entry = await self.async_set_unique_id(info["uuid"])
@@ -146,11 +177,12 @@ class ConfigFlow(config_entries.ConfigFlow, domain=DOMAIN):
         )
 
     async def async_step_zeroconf(
-        self, discovery_info: DiscoveryInfoType
+        self, discovery_info: zeroconf.ZeroconfServiceInfo
     ) -> FlowResult:
         """Handle zeroconf discovery."""
-        host = discovery_info["properties"].get("ip")
-        uuid = discovery_info["properties"].get("uuid")
+        properties = discovery_info.properties
+        host = properties.get("ip")
+        uuid = properties.get("uuid")
 
         if host is None or uuid is None:
             return self.async_abort(reason="unknown")
@@ -162,12 +194,12 @@ class ConfigFlow(config_entries.ConfigFlow, domain=DOMAIN):
         self._name = host
         self._input = {
             CONF_HOST: host,
-            CONF_PORT: discovery_info["properties"].get("port"),
+            CONF_PORT: properties.get("port"),
         }
 
         return await self.async_step_authenticate()
 
-    async def async_step_reauth(self, entry_data: ConfigType) -> FlowResult:
+    async def async_step_reauth(self, entry_data: Mapping[str, Any]) -> FlowResult:
         """Perform reauth upon an API authentication error."""
         self._name = entry_data[CONF_HOST]
         self._input = {
