@@ -4,7 +4,7 @@ from __future__ import annotations
 from collections.abc import Mapping
 from dataclasses import dataclass
 from datetime import timedelta
-from enum import IntEnum
+from enum import IntFlag
 import functools as ft
 import logging
 from typing import Any, final
@@ -21,8 +21,7 @@ from homeassistant.const import (
     SERVICE_TURN_ON,
     STATE_OFF,
     STATE_ON,
-    TEMP_CELSIUS,
-    TEMP_FAHRENHEIT,
+    UnitOfTemperature,
 )
 from homeassistant.core import HomeAssistant, ServiceCall
 import homeassistant.helpers.config_validation as cv
@@ -34,9 +33,7 @@ from homeassistant.helpers.entity import Entity, EntityDescription
 from homeassistant.helpers.entity_component import EntityComponent
 from homeassistant.helpers.temperature import display_temp as show_temp
 from homeassistant.helpers.typing import ConfigType
-from homeassistant.util.temperature import convert as convert_temperature
-
-# mypy: allow-untyped-defs, no-check-untyped-defs
+from homeassistant.util.unit_conversion import TemperatureConverter
 
 DEFAULT_MIN_TEMP = 110
 DEFAULT_MAX_TEMP = 140
@@ -58,7 +55,7 @@ STATE_HEAT_PUMP = "heat_pump"
 STATE_GAS = "gas"
 
 
-class WaterHeaterEntityFeature(IntEnum):
+class WaterHeaterEntityFeature(IntFlag):
     """Supported features of the fan entity."""
 
     TARGET_TEMPERATURE = 1
@@ -109,10 +106,12 @@ SET_OPERATION_MODE_SCHEMA = vol.Schema(
     }
 )
 
+# mypy: disallow-any-generics
+
 
 async def async_setup(hass: HomeAssistant, config: ConfigType) -> bool:
     """Set up water_heater devices."""
-    component = hass.data[DOMAIN] = EntityComponent(
+    component = hass.data[DOMAIN] = EntityComponent[WaterHeaterEntity](
         _LOGGER, DOMAIN, hass, SCAN_INTERVAL
     )
     await component.async_setup(config)
@@ -140,13 +139,13 @@ async def async_setup(hass: HomeAssistant, config: ConfigType) -> bool:
 
 async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
     """Set up a config entry."""
-    component: EntityComponent = hass.data[DOMAIN]
+    component: EntityComponent[WaterHeaterEntity] = hass.data[DOMAIN]
     return await component.async_setup_entry(entry)
 
 
 async def async_unload_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
     """Unload a config entry."""
-    component: EntityComponent = hass.data[DOMAIN]
+    component: EntityComponent[WaterHeaterEntity] = hass.data[DOMAIN]
     return await component.async_unload_entry(entry)
 
 
@@ -167,7 +166,7 @@ class WaterHeaterEntity(Entity):
     _attr_operation_list: list[str] | None = None
     _attr_precision: float
     _attr_state: None = None
-    _attr_supported_features: int
+    _attr_supported_features: WaterHeaterEntityFeature = WaterHeaterEntityFeature(0)
     _attr_target_temperature_high: float | None = None
     _attr_target_temperature_low: float | None = None
     _attr_target_temperature: float | None = None
@@ -184,15 +183,13 @@ class WaterHeaterEntity(Entity):
         """Return the precision of the system."""
         if hasattr(self, "_attr_precision"):
             return self._attr_precision
-        if self.hass.config.units.temperature_unit == TEMP_CELSIUS:
+        if self.hass.config.units.temperature_unit == UnitOfTemperature.CELSIUS:
             return PRECISION_TENTHS
         return PRECISION_WHOLE
 
     @property
     def capability_attributes(self) -> Mapping[str, Any]:
         """Return capability attributes."""
-        supported_features = self.supported_features or 0
-
         data: dict[str, Any] = {
             ATTR_MIN_TEMP: show_temp(
                 self.hass, self.min_temp, self.temperature_unit, self.precision
@@ -202,7 +199,7 @@ class WaterHeaterEntity(Entity):
             ),
         }
 
-        if supported_features & WaterHeaterEntityFeature.OPERATION_MODE:
+        if self.supported_features & WaterHeaterEntityFeature.OPERATION_MODE:
             data[ATTR_OPERATION_LIST] = self.operation_list
 
         return data
@@ -238,12 +235,10 @@ class WaterHeaterEntity(Entity):
             ),
         }
 
-        supported_features = self.supported_features or 0
-
-        if supported_features & WaterHeaterEntityFeature.OPERATION_MODE:
+        if self.supported_features & WaterHeaterEntityFeature.OPERATION_MODE:
             data[ATTR_OPERATION_MODE] = self.current_operation
 
-        if supported_features & WaterHeaterEntityFeature.AWAY_MODE:
+        if self.supported_features & WaterHeaterEntityFeature.AWAY_MODE:
             is_away = self.is_away_mode_on
             data[ATTR_AWAY_MODE] = STATE_ON if is_away else STATE_OFF
 
@@ -328,8 +323,8 @@ class WaterHeaterEntity(Entity):
         """Return the minimum temperature."""
         if hasattr(self, "_attr_min_temp"):
             return self._attr_min_temp
-        return convert_temperature(
-            DEFAULT_MIN_TEMP, TEMP_FAHRENHEIT, self.temperature_unit
+        return TemperatureConverter.convert(
+            DEFAULT_MIN_TEMP, UnitOfTemperature.FAHRENHEIT, self.temperature_unit
         )
 
     @property
@@ -337,10 +332,15 @@ class WaterHeaterEntity(Entity):
         """Return the maximum temperature."""
         if hasattr(self, "_attr_max_temp"):
             return self._attr_max_temp
-        return convert_temperature(
-            DEFAULT_MAX_TEMP, TEMP_FAHRENHEIT, self.temperature_unit
+        return TemperatureConverter.convert(
+            DEFAULT_MAX_TEMP, UnitOfTemperature.FAHRENHEIT, self.temperature_unit
         )
 
+    @property
+    def supported_features(self) -> WaterHeaterEntityFeature:
+        """Return the list of supported features."""
+        return self._attr_supported_features
+
 
 async def async_service_away_mode(
     entity: WaterHeaterEntity, service: ServiceCall
@@ -361,7 +361,7 @@ async def async_service_temperature_set(
 
     for value, temp in service.data.items():
         if value in CONVERTIBLE_ATTRIBUTE:
-            kwargs[value] = convert_temperature(
+            kwargs[value] = TemperatureConverter.convert(
                 temp, hass.config.units.temperature_unit, entity.temperature_unit
             )
         else:
