@@ -6,10 +6,12 @@ import sys
 from typing import Any
 
 import httpx
+from typing_extensions import Self
 
 from homeassistant.const import APPLICATION_NAME, EVENT_HOMEASSISTANT_CLOSE, __version__
 from homeassistant.core import Event, HomeAssistant, callback
 from homeassistant.loader import bind_hass
+from homeassistant.util.ssl import get_default_context, get_default_no_verify_context
 
 from .frame import warn_use
 
@@ -41,7 +43,7 @@ def get_async_client(hass: HomeAssistant, verify_ssl: bool = True) -> httpx.Asyn
 class HassHttpXAsyncClient(httpx.AsyncClient):
     """httpx AsyncClient that suppresses context management."""
 
-    async def __aenter__(self: HassHttpXAsyncClient) -> HassHttpXAsyncClient:
+    async def __aenter__(self) -> Self:
         """Prevent an integration from reopen of the client via context manager."""
         return self
 
@@ -63,15 +65,18 @@ def create_async_httpx_client(
 
     This method must be run in the event loop.
     """
+    ssl_context = (
+        get_default_context() if verify_ssl else get_default_no_verify_context()
+    )
     client = HassHttpXAsyncClient(
-        verify=verify_ssl,
+        verify=ssl_context,
         headers={USER_AGENT: SERVER_SOFTWARE},
         **kwargs,
     )
 
     original_aclose = client.aclose
 
-    client.aclose = warn_use(  # type: ignore[assignment]
+    client.aclose = warn_use(  # type: ignore[method-assign]
         client.aclose, "closes the Home Assistant httpx client"
     )
 
